﻿using System.Windows.Forms;

public class TabControlDnD : TabControl {

    private TabPage _sourceTabPage;

    /// <summary>
    /// Creates new instance.
    /// </summary>
    public TabControlDnD()
        : base() {
    }


    /// <summary>
    /// Raises the System.Windows.Forms.Control.MouseDown event.
    /// </summary>
    /// <param name="e">A System.Windows.Forms.MouseEventArgs that contains the event data.</param>
    protected override void OnMouseDown(MouseEventArgs e) {
        if ((e.Button == MouseButtons.Left) && (base.SelectedTab != null) && (!base.GetTabRect(base.SelectedIndex).IsEmpty)) {
            this._sourceTabPage = base.SelectedTab;
        }
        base.OnMouseDown(e);
    }

    /// <summary>
    /// Raises the System.Windows.Forms.Control.MouseMove event.
    /// </summary>
    /// <param name="e">A System.Windows.Forms.MouseEventArgs that contains the event data.</param>
    protected override void OnMouseMove(MouseEventArgs e) {
        if ((e.Button == MouseButtons.Left) && (this._sourceTabPage != null)) {
            var currTabPage = GetTabPageFromXY(e.X, e.Y);
            if (currTabPage != null) {
                if (base.TabPages.IndexOf(currTabPage) < base.TabPages.IndexOf(this._sourceTabPage)) {
                    base.Cursor = Cursors.PanWest;
                } else if (base.TabPages.IndexOf(currTabPage) > base.TabPages.IndexOf(this._sourceTabPage)) {
                    base.Cursor = Cursors.PanEast;
                } else { //tab is same as current
                    base.Cursor = Cursors.Default;
                }
            } else { //no tab selected
                base.Cursor = Cursors.No;
            }
        } else {
            base.Cursor = Cursors.Default;
        }
        base.OnMouseMove(e);
    }

    /// <summary>
    /// Raises the System.Windows.Forms.Control.MouseUp event.
    /// </summary>
    /// <param name="e">A System.Windows.Forms.MouseEventArgs that contains the event data.</param>
    protected override void OnMouseUp(MouseEventArgs e) {
        if ((e.Button == MouseButtons.Left) && (this._sourceTabPage != null)) {
            var currTabPage = GetTabPageFromXY(e.X, e.Y);
            if ((currTabPage != null) && (!currTabPage.Equals(this._sourceTabPage))) {
                if (base.TabPages.IndexOf(currTabPage) < base.TabPages.IndexOf(this._sourceTabPage)) {
                    base.TabPages.Remove(this._sourceTabPage);
                    base.TabPages.Insert(base.TabPages.IndexOf(currTabPage), this._sourceTabPage);
                    base.SelectedTab = this._sourceTabPage;
                } else if (base.TabPages.IndexOf(currTabPage) > base.TabPages.IndexOf(this._sourceTabPage)) {
                    base.TabPages.Remove(this._sourceTabPage);
                    base.TabPages.Insert(base.TabPages.IndexOf(currTabPage) + 1, this._sourceTabPage);
                    base.SelectedTab = this._sourceTabPage;
                }
            }
        }
        this._sourceTabPage = null;
        base.Cursor = Cursors.Default;
        base.OnMouseUp(e);
    }


    private TabPage GetTabPageFromXY(int x, int y) {
        for (int i = 0; i < base.TabPages.Count; ++i) {
            if (base.GetTabRect(i).Contains(x, y)) {
                return base.TabPages[i];
            }
        }
        return null;
    }

}
